% script FEM_force_springs

% USER PROVIDED INPUTS
% ______________________________________________

Nn = 5; % no. of nodes
Ne = 6; % no. of elements (also no. of internal forces for this problem)
kconstant = 120*ones(1,Ne); % spring stiffnesses
connect = [ 1 3; 3 4; 3 5; 3 5; 5 4; 4 2]; % connectivity matrix

% displacement and force boundary conditions will both be given as [node no.  value]
% or, more accurately, [global_unknown_number  value]

bcd = [ 2 0 ; 1 0]; % displacement boundary conditions - must go from high 
% to low node no.
Ndbc = size(bcd, 1); % determine no. of displacement bcs, Ndbc

% make sure the displacement bcs nodes are in descending order
% which is necessary when removing equilibrium equations later
for nn=1:Ndbc-1
if bcd(nn+1,1) > bcd(nn,1) 
error('displacement bc node numbers must be in descending order')
end
end

bcf = [5 20]; % force boundary conditions
Nfbc = size(bcf,1); % determine no. of force boundary conditions, Nfbc

% END OF USER PROVIDED INPUTS
% ________________________________________________
G = diag(1./kconstant, 0); % flexibility matrix
Ee = [-1;1]; % element equilibrium matrix

Eg = zeros(Nn, Ne); % initialize the global equilibrium matrix


% assemble the global equilibrium matrix
for nn = 1:Ne
nstart= connect(nn,1);
nend = connect(nn,2);
Eg(nstart, nn)=Eg(nstart,nn) + Ee(1,1);
Eg(nend,nn) = Eg(nend,nn) + Ee(2,1);
end
P = zeros(Nn,1);% initialize global force vector (will be modified)

% apply the force boundary conditions
for ii = 1: Nfbc
P(bcf(ii, 1))= bcf(ii,2);
end

% apply the displacement boundary conditions and obtain
% the reduced equilibrium matrix, E, and reduced loads vector, P, and matrix 
% of removed equations, Er, and specified displacements vector, Us.

Er = zeros(Ndbc,Ne); % initialize the matrix of removed equations
Us = zeros(Ndbc,1); % initialize a vector holding known displacements

% generate the reduced equilibrium matrix E, the reduced load vector P, and the
% matrix of removed equilibrium equations, Er, and generate a vector Us  with the 
% given displacements 
%-------changes
E=Eg;
for jj = 1:Ndbc
Er(jj,:)=Eg(bcd(jj,1),:);
Us(jj) = bcd(jj,2);
E(bcd(jj,1), :)= [ ];
P(bcd(jj,1)) = [ ];
% -------changes
end
% generate the compatibility matrix and compatibility equations
S = null(E).';
C = S*G; % multiply compatibility matrix by flexibility matrix
Ncomp =size(C,1); % determine the number of compatibility equations, Ncomp.
% Set up the combined system of modified equilibrium and compatibility
% equations and solve for the internal forces, F
Esys = [ E; C];
Psys = [P; S*Er'*Us];
F = Esys\Psys; % solve system for internal forces
Usys = Esys'\(G*F-Er'*Us); %solve for the system displacements, Usys
Uf =Usys(1:Ne - Ncomp); % extract the free displacements and put the
% free and specified displacements into global displacement vector Ug. The 
% number of free displacements is Ne - Ncomp since Ne is also the
% number of internal elongations (and internal forces) and the number of 
% compatibility equations is the number of relations between elongations
% when those elongations are expressed in terms of the free displacements

Ug = 1i*ones(Nn,1); % initialize Ug with all imaginary values
% replace imaginary values with the known displacements at specified nodes
for nn = 1:Ndbc
Ug(bcd(nn,1)) = bcd(nn,2);
end
% replace imaginary values remaining with free displacements, Uf
kk =1; % initialize counter
for jj= 1:Nn
if imag(Ug(jj)) == 1
Ug(jj) = Uf(kk);
kk = kk+1;
end
end
% determine reactions
R = Er*F;
% display internal forces, F , for all elements
disp("The internal spring forces are")
disp(F)
% display global displacements, Ug, at all nodes
disp("The global displacements are")
disp(Ug)

% display the reaction forces and their nodes
disp ("At the supports where the node numbers are")
node_numbers = bcd(1:Ndbc,1);
disp(node_numbers)
disp("the reactions are")
disp(R )
