% script FEM_stiffness_springs


% USER PROVIDED INPUTS
% __________________________________________
Nn = 5;  % number of  nodes (also number of global displacements)
Ne = 6;  % number of elements
% spring constants for the elements (must be of length Ne)
kconstant = 120*ones(1, Ne);
% connectivity matrix, where each row has the values
%  [start_node_number   end_node_number] . It must be a matrix of (Ne x 2) dimensions
connect = [ 1 3; 3 4; 3 5; 3 5; 5 4; 4 2];
% matrix of displacement boundary conditions, bcd, and matrix of 
% force boundary conditions, bcf, where each row contains
% [ global_unknown _number    value ]
bcd = [ 1 0; 2 0]; 
bcf = [ 5  20];
Nbcd = size(bcd,1);  % number of displacement boundary conditions
Nbcf = size(bcf,1); % number of force boundary conditions
% END OF USER PROVIDED INPUTS 
% _______________________________________________________
% initialize the element stiffness matrix, Ke,  and the global stiffness matrices, Kg, with % zeros. Also, initialize a matrix Kr to hold the rows of the global stiffness matrix that will % generate the reaction forces

Ke = zeros(2,2);
Kg = zeros(Nn , Nn);
Kr = zeros(Nbcd,Nn);
% Generate the element stiffness matrices and assemble into the global stiffness matrix
% Use the connectivity matrix to place the element stiffness 
% values in their correct positions

for nn = 1:Ne
ke = kconstant(nn); 		% element spring constant
Ke = [ ke -ke; -ke ke];
beg = connect(nn,1);    	% element beginning node
fin = connect(nn,2);    	% element finishing node
Kg(beg, beg) = Kg(beg,beg) + Ke(1,1);
Kg(beg, fin) = Kg(beg,fin) + Ke(1,2);
Kg(fin, beg) = Kg(fin,beg) + Ke(2,1);
Kg(fin,fin) = Kg(fin,fin) + Ke(2,2);
end
% initialize the force vector

P = zeros(Nn, 1);



% apply the displacement boundary conditions and save the rows needed to
% evaluate the reactions with the matrix Kr
 [maxK , i] = max(Kg(:));  % find largest stiffness
 %---------changes

for ii = 1:Nbcd
node =bcd(ii,1);
Kr(ii,:) = Kg(node,:);
Kg(node,node) = maxK*1.0E9;
P(node) = bcd(ii,2)*maxK*1.0E9;
end
%----changes
% apply the force boundary conditions
for ii = 1: Nbcf
node = bcf(ii,1);
P(node) = bcf(ii,2);
end

% solve the system of equations
u = Kg\P;

% evaluate internal forces in the springs
F = zeros(Ne,1); % initialize the force vector



for jj= 1:Ne
k = kconstant(jj);
start = connect(jj,1);
finish = connect(jj, 2);
F(jj) = k*(u(finish) - u(start));
end
% display all the results
disp ("The global displacements are:")
disp(u)

disp("The internal spring forces are:")
disp(F)
disp ("At the supports where the node numbers are")
node_numbers = bcd(1:Nbcd,1);
disp(node_numbers)
disp("the reactions are")
R = Kr*u;
disp(R)
