function r =R_loc(R, M, str)
% r = R_loc(R,M,str)solves rxR = M for the location of a single
% resultant force R that is statically equivalent to R and M.
% str is a string that must be either 'x=0' , 'y=0', or
% 'z=0', needed to determine a unique location for the resultant.
% R and M must be perpendicular to each other for a solution.
% the output, r, of the function is a vector containing the 
% (x,y,z) coordinates of the location of the resultant.

eR = R/norm(R);
eM = M/norm(M);
if abs(dot(eR,eM)) > 1.0e-3 % check for perpendicularity of R and M vectors
    warning('dot(eR,eM) >.001,likely no solution')
end
if strcmp(str, 'x=0')
    if abs(R(1))<eps
        error('Rx is zero, cannot choose x=0')
    else
        r = [ 0  -M(3)/R(1)  M(2)/R(1)];
    end
elseif strcmp(str,'y=0')
    if abs(R(2)) < eps
        error('Ry is zero, cannot choose y =0')
    else
        r = [ M(3)/R(2)  0  -M(1)/R(2)];
    end
elseif strcmp(str, 'z=0')
    if abs(R(3)) < eps
        error('Rz is zero, cannot choose z=0')
    else
        r = [ -M(2)/R(3)  M(1)/R(3)  0];
    end
else 
    error(' str must be ''x=0'' or ''y=0'' or ''z=0''')
end